@@ -102,7 +102,7 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2006 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2010 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -116,7 +116,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: zend_ini_parser.y,v 1.41.2.2 2006/04/12 09:51:54 dmitry Exp $ */
+/* $Id: zend_ini_parser.y 293154 2010-01-05 20:40:23Z sebastian $ */
 
 #define DEBUG_CFG_PARSER 0
 #include "zend.h"
@@ -157,11 +157,11 @@ void zend_ini_do_op(char type, zval *result, zval *op1, zval *op2)
 	int i_op1, i_op2;
 	char str_result[MAX_LENGTH_OF_LONG];
 
-	i_op1 = atoi(op1->value.str.val);
-	free(op1->value.str.val);
+	i_op1 = atoi(Z_STRVAL_P(op1));
+	free(Z_STRVAL_P(op1));
 	if (op2) {
-		i_op2 = atoi(op2->value.str.val);
-		free(op2->value.str.val);
+		i_op2 = atoi(Z_STRVAL_P(op2));
+		free(Z_STRVAL_P(op2));
 	} else {
 		i_op2 = 0;
 	}
@@ -184,30 +184,30 @@ void zend_ini_do_op(char type, zval *result, zval *op1, zval *op2)
 			break;
 	}
 
-	result->value.str.len = zend_sprintf(str_result, "%d", i_result);
-	result->value.str.val = (char *) malloc(result->value.str.len+1);
-	memcpy(result->value.str.val, str_result, result->value.str.len);
-	result->value.str.val[result->value.str.len] = 0;
-	result->type = IS_STRING;
+	Z_STRLEN_P(result) = zend_sprintf(str_result, "%d", i_result);
+	Z_STRVAL_P(result) = (char *) malloc(Z_STRLEN_P(result)+1);
+	memcpy(Z_STRVAL_P(result), str_result, Z_STRLEN_P(result));
+	Z_STRVAL_P(result)[Z_STRLEN_P(result)] = 0;
+	Z_TYPE_P(result) = IS_STRING;
 }
 
 void zend_ini_init_string(zval *result)
 {
-	result->value.str.val = malloc(1);
-	result->value.str.val[0] = 0;
-	result->value.str.len = 0;
-	result->type = IS_STRING;
+	Z_STRVAL_P(result) = malloc(1);
+	Z_STRVAL_P(result)[0] = 0;
+	Z_STRLEN_P(result) = 0;
+	Z_TYPE_P(result) = IS_STRING;
 }
 
 void zend_ini_add_string(zval *result, zval *op1, zval *op2)
-{           
-    int length = op1->value.str.len + op2->value.str.len;
+{
+	int length = Z_STRLEN_P(op1) + Z_STRLEN_P(op2);
 
-	result->value.str.val = (char *) realloc(op1->value.str.val, length+1);
-    memcpy(result->value.str.val+op1->value.str.len, op2->value.str.val, op2->value.str.len);
-    result->value.str.val[length] = 0;
-    result->value.str.len = length;
-    result->type = IS_STRING;
+	Z_STRVAL_P(result) = (char *) realloc(Z_STRVAL_P(op1), length+1);
+	memcpy(Z_STRVAL_P(result)+Z_STRLEN_P(op1), Z_STRVAL_P(op2), Z_STRLEN_P(op2));
+	Z_STRVAL_P(result)[length] = 0;
+	Z_STRLEN_P(result) = length;
+	Z_TYPE_P(result) = IS_STRING;
 }
 
 void zend_ini_get_constant(zval *result, zval *name)
@@ -215,15 +215,15 @@ void zend_ini_get_constant(zval *result, zval *name)
 	zval z_constant;
 	TSRMLS_FETCH();
 
-	if (!memchr(name->value.str.val, ':', name->value.str.len)
-		   	&& zend_get_constant(name->value.str.val, name->value.str.len, &z_constant TSRMLS_CC)) {
+	if (!memchr(Z_STRVAL_P(name), ':', Z_STRLEN_P(name))
+		   	&& zend_get_constant(Z_STRVAL_P(name), Z_STRLEN_P(name), &z_constant TSRMLS_CC)) {
 		/* z_constant is emalloc()'d */
 		convert_to_string(&z_constant);
-		result->value.str.val = zend_strndup(z_constant.value.str.val, z_constant.value.str.len);
-		result->value.str.len = z_constant.value.str.len;
-		result->type = z_constant.type;
+		Z_STRVAL_P(result) = zend_strndup(Z_STRVAL(z_constant), Z_STRLEN(z_constant));
+		Z_STRLEN_P(result) = Z_STRLEN(z_constant);
+		Z_TYPE_P(result) = Z_TYPE(z_constant);
 		zval_dtor(&z_constant);
-		free(name->value.str.val);	
+		free(Z_STRVAL_P(name));
 	} else {
 		*result = *name;
 	}
@@ -235,13 +235,13 @@ void zend_ini_get_var(zval *result, zval *name)
 	char *envvar;
 	TSRMLS_FETCH();
 
-	if (zend_get_configuration_directive(name->value.str.val, name->value.str.len+1, &curval) == SUCCESS) {
-		result->value.str.val = zend_strndup(curval.value.str.val, curval.value.str.len);
-		result->value.str.len = curval.value.str.len;
-	} else if ((envvar = zend_getenv(name->value.str.val, name->value.str.len TSRMLS_CC)) != NULL ||
-			   (envvar = getenv(name->value.str.val)) != NULL) {
-		result->value.str.val = strdup(envvar);
-		result->value.str.len = strlen(envvar);
+	if (zend_get_configuration_directive(Z_STRVAL_P(name), Z_STRLEN_P(name)+1, &curval) == SUCCESS) {
+		Z_STRVAL_P(result) = zend_strndup(Z_STRVAL(curval), Z_STRLEN(curval));
+		Z_STRLEN_P(result) = Z_STRLEN(curval);
+	} else if ((envvar = zend_getenv(Z_STRVAL_P(name), Z_STRLEN_P(name) TSRMLS_CC)) != NULL ||
+			   (envvar = getenv(Z_STRVAL_P(name))) != NULL) {
+		Z_STRVAL_P(result) = strdup(envvar);
+		Z_STRLEN_P(result) = strlen(envvar);
 	} else {
 		zend_ini_init_string(result);
 	}
@@ -256,10 +256,14 @@ static void ini_error(char *str)
 	TSRMLS_FETCH();
 
 	currently_parsed_filename = zend_ini_scanner_get_filename(TSRMLS_C);
-	error_buf_len = 128+strlen(currently_parsed_filename); /* should be more than enough */
-	error_buf = (char *) emalloc(error_buf_len);
+	if (currently_parsed_filename) {
+		error_buf_len = 128+strlen(currently_parsed_filename); /* should be more than enough */
+		error_buf = (char *) emalloc(error_buf_len);
 
-	sprintf(error_buf, "Error parsing %s on line %d\n", currently_parsed_filename, zend_ini_scanner_get_lineno(TSRMLS_C));
+		sprintf(error_buf, "Error parsing %s on line %d\n", currently_parsed_filename, zend_ini_scanner_get_lineno(TSRMLS_C));
+	} else {
+		error_buf = estrdup("Invalid configuration directive\n");
+	}
 
 	if (CG(ini_parser_unbuffered_errors)) {
 #ifdef PHP_WIN32
@@ -301,6 +305,29 @@ ZEND_API int zend_parse_ini_file(zend_file_handle *fh, zend_bool unbuffered_erro
 }
 
 
+ZEND_API int zend_parse_ini_string(char *str, zend_bool unbuffered_errors, zend_ini_parser_cb_t ini_parser_cb, void *arg)
+{
+	zend_ini_parser_param ini_parser_param;
+	TSRMLS_FETCH();
+
+	ini_parser_param.ini_parser_cb = ini_parser_cb;
+	ini_parser_param.arg = arg;
+
+	CG(ini_parser_param) = &ini_parser_param;
+	if (zend_ini_prepare_string_for_scanning(str TSRMLS_CC)==FAILURE) {
+		return FAILURE;
+	}
+
+	CG(ini_parser_unbuffered_errors) = unbuffered_errors;
+
+	if (ini_parse(TSRMLS_C)) {
+		return SUCCESS;
+	} else {
+		return FAILURE;
+	}
+}
+
+
 
 
 /* Enabling traces.  */
@@ -625,9 +652,9 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   221,   221,   222,   226,   234,   242,   243,   244,   249,
-     250,   251,   252,   253,   258,   259,   260,   261,   262,   263,
-     267,   271,   272,   273,   274,   275,   276,   280
+       0,   248,   248,   249,   253,   261,   269,   270,   271,   276,
+     277,   278,   279,   280,   285,   286,   287,   288,   289,   290,
+     294,   298,   299,   300,   301,   302,   303,   307
 };
 #endif
 
@@ -1552,11 +1579,11 @@ yyreduce:
 
     {
 #if DEBUG_CFG_PARSER
-			printf("'%s' = '%s'\n", (yyvsp[(1) - (3)]).value.str.val, (yyvsp[(3) - (3)]).value.str.val);
+			printf("'%s' = '%s'\n", Z_STRVAL((yyvsp[(1) - (3)])), Z_STRVAL((yyvsp[(3) - (3)])));
 #endif
 			ZEND_INI_PARSER_CB(&(yyvsp[(1) - (3)]), &(yyvsp[(3) - (3)]), ZEND_INI_PARSER_ENTRY, ZEND_INI_PARSER_ARG);
-			free((yyvsp[(1) - (3)]).value.str.val);
-			free((yyvsp[(3) - (3)]).value.str.val);
+			free(Z_STRVAL((yyvsp[(1) - (3)])));
+			free(Z_STRVAL((yyvsp[(3) - (3)])));
 		}
     break;
 
@@ -1564,22 +1591,22 @@ yyreduce:
 
     {
 #if DEBUG_CFG_PARSER
-			printf("'%s'[ ] = '%s'\n", (yyvsp[(1) - (4)]).value.str.val, (yyvsp[(4) - (4)]).value.str.val);
+			printf("'%s'[ ] = '%s'\n", Z_STRVAL((yyvsp[(1) - (4)])), Z_STRVAL((yyvsp[(4) - (4)])));
 #endif
 			ZEND_INI_PARSER_CB(&(yyvsp[(1) - (4)]), &(yyvsp[(4) - (4)]), ZEND_INI_PARSER_POP_ENTRY, ZEND_INI_PARSER_ARG);
-			free((yyvsp[(1) - (4)]).value.str.val);
-			free((yyvsp[(4) - (4)]).value.str.val);
+			free(Z_STRVAL((yyvsp[(1) - (4)])));
+			free(Z_STRVAL((yyvsp[(4) - (4)])));
 		}
     break;
 
   case 6:
 
-    { ZEND_INI_PARSER_CB(&(yyvsp[(1) - (1)]), NULL, ZEND_INI_PARSER_ENTRY, ZEND_INI_PARSER_ARG); free((yyvsp[(1) - (1)]).value.str.val); }
+    { ZEND_INI_PARSER_CB(&(yyvsp[(1) - (1)]), NULL, ZEND_INI_PARSER_ENTRY, ZEND_INI_PARSER_ARG); free(Z_STRVAL((yyvsp[(1) - (1)]))); }
     break;
 
   case 7:
 
-    { ZEND_INI_PARSER_CB(&(yyvsp[(1) - (1)]), NULL, ZEND_INI_PARSER_SECTION, ZEND_INI_PARSER_ARG); free((yyvsp[(1) - (1)]).value.str.val); }
+    { ZEND_INI_PARSER_CB(&(yyvsp[(1) - (1)]), NULL, ZEND_INI_PARSER_SECTION, ZEND_INI_PARSER_ARG); free(Z_STRVAL((yyvsp[(1) - (1)]))); }
     break;
 
   case 9:
@@ -1629,7 +1656,7 @@ yyreduce:
 
   case 18:
 
-    { zend_ini_add_string(&(yyval), &(yyvsp[(1) - (2)]), &(yyvsp[(2) - (2)])); free((yyvsp[(2) - (2)]).value.str.val); }
+    { zend_ini_add_string(&(yyval), &(yyvsp[(1) - (2)]), &(yyvsp[(2) - (2)])); free(Z_STRVAL((yyvsp[(2) - (2)]))); }
     break;
 
   case 19:
