@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2004 The PHP Group                                |
+   | Copyright (c) 1997-2010 The PHP Group                                |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 3.0 of the PHP license,       |
+   | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_0.txt.                                  |
+   | http://www.php.net/license/3_01.txt                                  |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -37,48 +37,282 @@
 /* True global resources - no need for thread safety here */
 static int le_gmp;
 
+/* {{{ arginfo */
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_gmp_init, 0, 0, 1)
+	ZEND_ARG_INFO(0, number)
+	ZEND_ARG_INFO(0, base)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_intval, 0)
+	ZEND_ARG_INFO(0, gmpnumber)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_gmp_strval, 0, 0, 1)
+	ZEND_ARG_INFO(0, gmpnumber)
+	ZEND_ARG_INFO(0, base)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_add, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_sub, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_mul, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_gmp_div_qr, 0, 0, 2)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+	ZEND_ARG_INFO(0, round)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_gmp_div_r, 0, 0, 2)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+	ZEND_ARG_INFO(0, round)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_gmp_div_q, 0, 0, 2)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+	ZEND_ARG_INFO(0, round)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_mod, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_divexact, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_neg, 0)
+	ZEND_ARG_INFO(0, a)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_abs, 0)
+	ZEND_ARG_INFO(0, a)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_fact, 0)
+	ZEND_ARG_INFO(0, a)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_pow, 0)
+	ZEND_ARG_INFO(0, base)
+	ZEND_ARG_INFO(0, exp)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_powm, 0)
+	ZEND_ARG_INFO(0, base)
+	ZEND_ARG_INFO(0, exp)
+	ZEND_ARG_INFO(0, mod)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_sqrt, 0)
+	ZEND_ARG_INFO(0, a)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_sqrtrem, 0)
+	ZEND_ARG_INFO(0, a)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_perfect_square, 0)
+	ZEND_ARG_INFO(0, a)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_gmp_prob_prime, 0, 0, 1)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, reps)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_gcd, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_gcdext, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_invert, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_jacobi, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_legendre, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_cmp, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_sign, 0)
+	ZEND_ARG_INFO(0, a)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_gmp_random, 0, 0, 0)
+	ZEND_ARG_INFO(0, limiter)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_and, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_or, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_com, 0)
+	ZEND_ARG_INFO(0, a)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_xor, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO_EX(arginfo_gmp_setbit, 0, 0, 2)
+	ZEND_ARG_INFO(1, a)
+	ZEND_ARG_INFO(0, index)
+	ZEND_ARG_INFO(0, set_clear)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_clrbit, 0)
+	ZEND_ARG_INFO(1, a)
+	ZEND_ARG_INFO(0, index)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_popcount, 0)
+	ZEND_ARG_INFO(0, a)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_hamdist, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, b)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_scan0, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, start)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_scan1, 0)
+	ZEND_ARG_INFO(0, a)
+	ZEND_ARG_INFO(0, start)
+ZEND_END_ARG_INFO()
+
+static
+ZEND_BEGIN_ARG_INFO(arginfo_gmp_nextprime, 0)
+	ZEND_ARG_INFO(0, a)
+ZEND_END_ARG_INFO()
+
+/* }}} */
+
+ZEND_DECLARE_MODULE_GLOBALS(gmp)
+static ZEND_GINIT_FUNCTION(gmp);
+
 /* {{{ gmp_functions[]
  */
-function_entry gmp_functions[] = {
-	ZEND_FE(gmp_init,	NULL)
-	ZEND_FE(gmp_intval,	NULL)
-	ZEND_FE(gmp_strval,	NULL)
-	ZEND_FE(gmp_add,	    NULL)
-	ZEND_FE(gmp_sub,	    NULL)
-	ZEND_FE(gmp_mul,	    NULL)
-	ZEND_FE(gmp_div_qr,	NULL)
-	ZEND_FE(gmp_div_q,	NULL)
-	ZEND_FE(gmp_div_r,	NULL)
-	ZEND_FALIAS(gmp_div,	gmp_div_q, NULL)
-	ZEND_FE(gmp_mod,	NULL)
-	ZEND_FE(gmp_divexact,	NULL)
-	ZEND_FE(gmp_neg,	NULL)
-	ZEND_FE(gmp_abs,	NULL)
-	ZEND_FE(gmp_fact,	NULL)
-	ZEND_FE(gmp_sqrt,	NULL)
-	ZEND_FE(gmp_sqrtrem,	NULL)
-	ZEND_FE(gmp_pow,	NULL)
-	ZEND_FE(gmp_powm,	NULL)
-	ZEND_FE(gmp_perfect_square,	NULL)
-	ZEND_FE(gmp_prob_prime,	NULL)
-	ZEND_FE(gmp_gcd,	NULL)
-	ZEND_FE(gmp_gcdext,	NULL)
-	ZEND_FE(gmp_invert,	NULL)
-	ZEND_FE(gmp_jacobi,	NULL)
-	ZEND_FE(gmp_legendre,	NULL)
-	ZEND_FE(gmp_cmp,	NULL)
-	ZEND_FE(gmp_sign,	NULL)
-	ZEND_FE(gmp_random,	NULL)
-	ZEND_FE(gmp_and,	NULL)
-	ZEND_FE(gmp_or,	NULL)
-	ZEND_FE(gmp_com,	NULL)
-	ZEND_FE(gmp_xor,	NULL)
-	ZEND_FE(gmp_setbit,	first_arg_force_ref)
-	ZEND_FE(gmp_clrbit,	first_arg_force_ref)
-	ZEND_FE(gmp_scan0, NULL)
-	ZEND_FE(gmp_scan1, NULL)
-	ZEND_FE(gmp_popcount, NULL)
-	ZEND_FE(gmp_hamdist, NULL)
+zend_function_entry gmp_functions[] = {
+	ZEND_FE(gmp_init,	arginfo_gmp_init)
+	ZEND_FE(gmp_intval,	arginfo_gmp_intval)
+	ZEND_FE(gmp_strval,	arginfo_gmp_strval)
+	ZEND_FE(gmp_add,	    arginfo_gmp_add)
+	ZEND_FE(gmp_sub,	    arginfo_gmp_sub)
+	ZEND_FE(gmp_mul,	    arginfo_gmp_mul)
+	ZEND_FE(gmp_div_qr,	arginfo_gmp_div_qr)
+	ZEND_FE(gmp_div_q,	arginfo_gmp_div_q)
+	ZEND_FE(gmp_div_r,	arginfo_gmp_div_r)
+	ZEND_FALIAS(gmp_div,	gmp_div_q, arginfo_gmp_div_q)
+	ZEND_FE(gmp_mod,	arginfo_gmp_mod)
+	ZEND_FE(gmp_divexact,	arginfo_gmp_divexact)
+	ZEND_FE(gmp_neg,	arginfo_gmp_neg)
+	ZEND_FE(gmp_abs,	arginfo_gmp_abs)
+	ZEND_FE(gmp_fact,	arginfo_gmp_fact)
+	ZEND_FE(gmp_sqrt,	arginfo_gmp_sqrt)
+	ZEND_FE(gmp_sqrtrem,	arginfo_gmp_sqrtrem)
+	ZEND_FE(gmp_pow,	arginfo_gmp_pow)
+	ZEND_FE(gmp_powm,	arginfo_gmp_powm)
+	ZEND_FE(gmp_perfect_square,	arginfo_gmp_perfect_square)
+	ZEND_FE(gmp_prob_prime,	arginfo_gmp_prob_prime)
+	ZEND_FE(gmp_gcd,	arginfo_gmp_gcd)
+	ZEND_FE(gmp_gcdext,	arginfo_gmp_gcdext)
+	ZEND_FE(gmp_invert,	arginfo_gmp_invert)
+	ZEND_FE(gmp_jacobi,	arginfo_gmp_jacobi)
+	ZEND_FE(gmp_legendre,	arginfo_gmp_legendre)
+	ZEND_FE(gmp_cmp,	arginfo_gmp_cmp)
+	ZEND_FE(gmp_sign,	arginfo_gmp_sign)
+	ZEND_FE(gmp_random,	arginfo_gmp_random)
+	ZEND_FE(gmp_and,	arginfo_gmp_and)
+	ZEND_FE(gmp_or,	arginfo_gmp_or)
+	ZEND_FE(gmp_com,	arginfo_gmp_com)
+	ZEND_FE(gmp_xor,	arginfo_gmp_xor)
+	ZEND_FE(gmp_setbit,	arginfo_gmp_setbit)
+	ZEND_FE(gmp_clrbit,	arginfo_gmp_clrbit)
+	ZEND_FE(gmp_scan0, arginfo_gmp_scan0)
+	ZEND_FE(gmp_scan1, arginfo_gmp_scan1)
+	ZEND_FE(gmp_popcount, arginfo_gmp_popcount)
+	ZEND_FE(gmp_hamdist, arginfo_gmp_hamdist)
+	ZEND_FE(gmp_nextprime, arginfo_gmp_nextprime)
 	{NULL, NULL, NULL}	/* Must be the last line in gmp_functions[] */
 };
 /* }}} */
@@ -90,22 +324,21 @@ zend_module_entry gmp_module_entry = {
 	"gmp",
 	gmp_functions,
 	ZEND_MODULE_STARTUP_N(gmp),
-	ZEND_MODULE_SHUTDOWN_N(gmp),
+	NULL,
 	NULL,
 	ZEND_MODULE_DEACTIVATE_N(gmp),
 	ZEND_MODULE_INFO_N(gmp),
 	NO_VERSION_YET,
-	STANDARD_MODULE_PROPERTIES
+	ZEND_MODULE_GLOBALS(gmp),
+	ZEND_GINIT(gmp),
+	NULL,
+	NULL,
+	STANDARD_MODULE_PROPERTIES_EX
 };
 /* }}} */
 
-ZEND_DECLARE_MODULE_GLOBALS(gmp)
-
 #ifdef COMPILE_DL_GMP
 ZEND_GET_MODULE(gmp)
-# ifdef PHP_WIN32
-# include "zend_arg_defs.c"
-# endif
 #endif
 
 static void _php_gmpnum_free(zend_rsrc_list_entry *rsrc TSRMLS_DC);
@@ -140,9 +373,9 @@ static void gmp_efree(void *ptr, size_t size)
 }
 /* }}} */
 
-/* {{{ php_gmp_init_globals
+/* {{{ ZEND_GINIT_FUNCTION
  */
-static void php_gmp_init_globals(zend_gmp_globals *gmp_globals)
+static ZEND_GINIT_FUNCTION(gmp)
 {
 	gmp_globals->rand_initialized = 0;
 }
@@ -152,12 +385,11 @@ static void php_gmp_init_globals(zend_gmp_globals *gmp_globals)
  */
 ZEND_MODULE_STARTUP_D(gmp)
 {
-	ZEND_INIT_MODULE_GLOBALS(gmp, php_gmp_init_globals, NULL);
-
 	le_gmp = zend_register_list_destructors_ex(_php_gmpnum_free, NULL, GMP_RESOURCE_NAME, module_number);
 	REGISTER_LONG_CONSTANT("GMP_ROUND_ZERO", GMP_ROUND_ZERO, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("GMP_ROUND_PLUSINF", GMP_ROUND_PLUSINF, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("GMP_ROUND_MINUSINF", GMP_ROUND_MINUSINF, CONST_CS | CONST_PERSISTENT);
+	REGISTER_STRING_CONSTANT("GMP_VERSION", (char *)gmp_version, CONST_CS | CONST_PERSISTENT);
 
 	mp_set_memory_functions(gmp_emalloc, gmp_erealloc, gmp_efree);
 
@@ -178,25 +410,14 @@ ZEND_MODULE_DEACTIVATE_D(gmp)
 }
 /* }}} */
 
-/* {{{ ZEND_MSHUTDOWN_FUNCTION
- */
-ZEND_MODULE_SHUTDOWN_D(gmp)
-{
-	return SUCCESS;
-}
-/* }}} */
-
 /* {{{ ZEND_MINFO_FUNCTION
  */
 ZEND_MODULE_INFO_D(gmp)
 {
 	php_info_print_table_start();
 	php_info_print_table_row(2, "gmp support", "enabled");
+	php_info_print_table_row(2, "GMP version", gmp_version);
 	php_info_print_table_end();
-
-	/* Remove comments if you have entries in php.ini
-	DISPLAY_INI_ENTRIES();
-	*/
 }
 /* }}} */
 
@@ -282,8 +503,8 @@ typedef void (*gmp_binary_op2_t)(mpz_ptr, mpz_ptr, mpz_srcptr, mpz_srcptr);
 typedef unsigned long (*gmp_binary_ui_op2_t)(mpz_ptr, mpz_ptr, mpz_srcptr, unsigned long);
 /* }}} */
 
-#define gmp_zval_binary_ui_op(r, a, b, o, u) gmp_zval_binary_ui_op_ex(r, a, b, o, u, 0 TSRMLS_CC)
-#define gmp_zval_binary_ui_op2(r, a, b, o, u) gmp_zval_binary_ui_op2_ex(r, a, b, o, u, 0 TSRMLS_CC)
+#define gmp_zval_binary_ui_op(r, a, b, o, u) gmp_zval_binary_ui_op_ex(r, a, b, o, u, 0, 0 TSRMLS_CC)
+#define gmp_zval_binary_ui_op2(r, a, b, o, u) gmp_zval_binary_ui_op2_ex(r, a, b, o, u, 0, 0 TSRMLS_CC)
 
 #define gmp_binary_ui_op(op, uop) _gmp_binary_ui_op(INTERNAL_FUNCTION_PARAM_PASSTHRU, op, uop)
 #define gmp_binary_op(op)         _gmp_binary_ui_op(INTERNAL_FUNCTION_PARAM_PASSTHRU, op, NULL)
@@ -298,7 +519,7 @@ typedef unsigned long (*gmp_binary_ui_op2_t)(mpz_ptr, mpz_ptr, mpz_srcptr, unsig
    Execute GMP binary operation.
    May return GMP resource or long if operation allows this
 */
-static inline void gmp_zval_binary_ui_op_ex(zval *return_value, zval **a_arg, zval **b_arg, gmp_binary_op_t gmp_op, gmp_binary_ui_op_t gmp_ui_op, int allow_ui_return TSRMLS_DC) 
+static inline void gmp_zval_binary_ui_op_ex(zval *return_value, zval **a_arg, zval **b_arg, gmp_binary_op_t gmp_op, gmp_binary_ui_op_t gmp_ui_op, int allow_ui_return, int check_b_zero TSRMLS_DC) 
 {
 	mpz_t *gmpnum_a, *gmpnum_b, *gmpnum_result;
 	unsigned long long_result = 0;
@@ -312,8 +533,18 @@ static inline void gmp_zval_binary_ui_op_ex(zval *return_value, zval **a_arg, zv
 		FETCH_GMP_ZVAL(gmpnum_b, b_arg);
 	}
 
-	if (!Z_LVAL_PP(b_arg)) {
-		RETURN_FALSE;
+	if(check_b_zero) {
+		int b_is_zero = 0;
+		if(use_ui) {
+			b_is_zero = (Z_LVAL_PP(b_arg) == 0);
+		} else {
+			b_is_zero = !mpz_cmp_ui(*gmpnum_b, 0);
+		}
+
+		if(b_is_zero) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Zero operand not allowed");
+			RETURN_FALSE;
+		}
 	}
 
 	INIT_GMP_NUM(gmpnum_result);
@@ -341,7 +572,7 @@ static inline void gmp_zval_binary_ui_op_ex(zval *return_value, zval **a_arg, zv
    Execute GMP binary operation which returns 2 values.
    May return GMP resources or longs if operation allows this.
 */
-static inline void gmp_zval_binary_ui_op2_ex(zval *return_value, zval **a_arg, zval **b_arg, gmp_binary_op2_t gmp_op, gmp_binary_ui_op2_t gmp_ui_op, int allow_ui_return TSRMLS_DC)
+static inline void gmp_zval_binary_ui_op2_ex(zval *return_value, zval **a_arg, zval **b_arg, gmp_binary_op2_t gmp_op, gmp_binary_ui_op2_t gmp_ui_op, int allow_ui_return, int check_b_zero TSRMLS_DC)
 {
 	mpz_t *gmpnum_a, *gmpnum_b, *gmpnum_result1, *gmpnum_result2;
 	zval r;
@@ -357,8 +588,18 @@ static inline void gmp_zval_binary_ui_op2_ex(zval *return_value, zval **a_arg, z
 		FETCH_GMP_ZVAL(gmpnum_b, b_arg);
 	}
 
-	if (!Z_LVAL_PP(b_arg)) {
-		RETURN_FALSE;
+	if(check_b_zero) {
+		int b_is_zero = 0;
+		if(use_ui) {
+			b_is_zero = (Z_LVAL_PP(b_arg) == 0);
+		} else {
+			b_is_zero = !mpz_cmp_ui(*gmpnum_b, 0);
+		}
+
+		if(b_is_zero) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Zero operand not allowed");
+			RETURN_FALSE;
+		}
 	}
 
 	INIT_GMP_NUM(gmpnum_result1);
@@ -653,13 +894,13 @@ ZEND_FUNCTION(gmp_div_qr)
 
 	switch (round) {
 	case GMP_ROUND_ZERO:
-		gmp_zval_binary_ui_op2(return_value, a_arg, b_arg, mpz_tdiv_qr, (gmp_binary_ui_op2_t)mpz_tdiv_qr_ui);
+		gmp_zval_binary_ui_op2_ex(return_value, a_arg, b_arg, mpz_tdiv_qr, (gmp_binary_ui_op2_t)mpz_tdiv_qr_ui, 0, 1 TSRMLS_CC);
 		break;
 	case GMP_ROUND_PLUSINF:
-		gmp_zval_binary_ui_op2(return_value, a_arg, b_arg, mpz_cdiv_qr, (gmp_binary_ui_op2_t)mpz_cdiv_qr_ui);
+		gmp_zval_binary_ui_op2_ex(return_value, a_arg, b_arg, mpz_cdiv_qr, (gmp_binary_ui_op2_t)mpz_cdiv_qr_ui, 0, 1 TSRMLS_CC);
 		break;
 	case GMP_ROUND_MINUSINF:
-		gmp_zval_binary_ui_op2(return_value, a_arg, b_arg, mpz_fdiv_qr, (gmp_binary_ui_op2_t)mpz_fdiv_qr_ui);
+		gmp_zval_binary_ui_op2_ex(return_value, a_arg, b_arg, mpz_fdiv_qr, (gmp_binary_ui_op2_t)mpz_fdiv_qr_ui, 0, 1 TSRMLS_CC);
 		break;
 	}
 							   
@@ -690,13 +931,13 @@ ZEND_FUNCTION(gmp_div_r)
 
 	switch (round) {
 	case GMP_ROUND_ZERO:
-		gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_tdiv_r, (gmp_binary_ui_op_t)mpz_tdiv_r_ui, 1 TSRMLS_CC);
+		gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_tdiv_r, (gmp_binary_ui_op_t)mpz_tdiv_r_ui, 1, 1 TSRMLS_CC);
 		break;
 	case GMP_ROUND_PLUSINF:
-		gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_cdiv_r, (gmp_binary_ui_op_t)mpz_cdiv_r_ui, 1 TSRMLS_CC);
+		gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_cdiv_r, (gmp_binary_ui_op_t)mpz_cdiv_r_ui, 1, 1 TSRMLS_CC);
 		break;
 	case GMP_ROUND_MINUSINF:
-		gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_fdiv_r, (gmp_binary_ui_op_t)mpz_fdiv_r_ui, 1 TSRMLS_CC);
+		gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_fdiv_r, (gmp_binary_ui_op_t)mpz_fdiv_r_ui, 1, 1 TSRMLS_CC);
 		break;
 	}
 }
@@ -726,13 +967,13 @@ ZEND_FUNCTION(gmp_div_q)
 
 	switch (round) {
 	case GMP_ROUND_ZERO:
-		gmp_zval_binary_ui_op(return_value, a_arg, b_arg, mpz_tdiv_q, (gmp_binary_ui_op_t)mpz_tdiv_q_ui);
+		gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_tdiv_q, (gmp_binary_ui_op_t)mpz_tdiv_q_ui, 0, 1 TSRMLS_CC);
 		break;
 	case GMP_ROUND_PLUSINF:
-		gmp_zval_binary_ui_op(return_value, a_arg, b_arg, mpz_cdiv_q, (gmp_binary_ui_op_t)mpz_cdiv_q_ui);
+		gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_cdiv_q, (gmp_binary_ui_op_t)mpz_cdiv_q_ui, 0, 1 TSRMLS_CC);
 		break;
 	case GMP_ROUND_MINUSINF:
-		gmp_zval_binary_ui_op(return_value, a_arg, b_arg, mpz_fdiv_q, (gmp_binary_ui_op_t)mpz_fdiv_q_ui);
+		gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_fdiv_q, (gmp_binary_ui_op_t)mpz_fdiv_q_ui, 0, 1 TSRMLS_CC);
 		break;
 	}
 							   
@@ -749,7 +990,7 @@ ZEND_FUNCTION(gmp_mod)
 		WRONG_PARAM_COUNT;
 	}
 
-	gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_mod, (gmp_binary_ui_op_t)mpz_mod_ui, 1 TSRMLS_CC);
+	gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_mod, (gmp_binary_ui_op_t)mpz_mod_ui, 1, 1 TSRMLS_CC);
 }
 /* }}} */
 
@@ -757,7 +998,13 @@ ZEND_FUNCTION(gmp_mod)
    Divide a by b using exact division algorithm */
 ZEND_FUNCTION(gmp_divexact)
 {
-	gmp_binary_op(mpz_divexact);
+	zval **a_arg, **b_arg;
+
+	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &a_arg, &b_arg) == FAILURE){
+		WRONG_PARAM_COUNT;
+	}
+	
+	gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_divexact, NULL, 0, 1 TSRMLS_CC);
 }
 /* }}} */
 
@@ -781,7 +1028,28 @@ ZEND_FUNCTION(gmp_abs)
    Calculates factorial function */
 ZEND_FUNCTION(gmp_fact)
 {
-	gmp_unary_ui_op(mpz_fac_ui);
+	zval **a_arg;
+	mpz_t *gmpnum_tmp;
+
+	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &a_arg) == FAILURE){
+		WRONG_PARAM_COUNT;
+	}
+
+	if (Z_TYPE_PP(a_arg) == IS_RESOURCE) {
+		FETCH_GMP_ZVAL(gmpnum_tmp, a_arg);
+		if (mpz_sgn(*gmpnum_tmp) < 0) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING,"Number has to be greater than or equal to 0");
+			RETURN_FALSE;
+		}
+	} else {
+		convert_to_long_ex(a_arg);
+		if (Z_LVAL_PP(a_arg) < 0) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING,"Number has to be greater than or equal to 0");
+			RETURN_FALSE;
+		}
+	}
+		
+	gmp_zval_unary_ui_op(return_value, a_arg, mpz_fac_ui);
 }
 /* }}} */
 
@@ -839,6 +1107,10 @@ ZEND_FUNCTION(gmp_powm)
 		use_ui = 1;
 	} else {
 		FETCH_GMP_ZVAL(gmpnum_exp, exp_arg);
+		if (mpz_sgn(*gmpnum_exp) < 0) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING,"Second parameter cannot be less than 0");
+			RETURN_FALSE;
+		}
 	}
 	FETCH_GMP_ZVAL(gmpnum_mod, mod_arg);
 
@@ -862,7 +1134,24 @@ ZEND_FUNCTION(gmp_powm)
    Takes integer part of square root of a */
 ZEND_FUNCTION(gmp_sqrt)
 {
-	gmp_unary_op(mpz_sqrt);
+	zval **a_arg;
+	mpz_t *gmpnum_a, *gmpnum_result;
+
+	if (ZEND_NUM_ARGS() != 1 || zend_get_parameters_ex(1, &a_arg) == FAILURE){
+		WRONG_PARAM_COUNT;
+	}
+	
+	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
+
+	if (mpz_sgn(*gmpnum_a) < 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING,"Number has to be greater than or equal to 0");
+		RETURN_FALSE;
+	}	
+	
+	INIT_GMP_NUM(gmpnum_result);
+	mpz_sqrt(*gmpnum_result, *gmpnum_a);
+
+	ZEND_REGISTER_RESOURCE(return_value, gmpnum_result, le_gmp);
 }
 /* }}} */
 
@@ -879,7 +1168,12 @@ ZEND_FUNCTION(gmp_sqrtrem)
 	}
 
 	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
-
+	
+	if (mpz_sgn(*gmpnum_a) < 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING,"Number has to be greater than or equal to 0");
+		RETURN_FALSE;
+	}
+	
 	INIT_GMP_NUM(gmpnum_result1);
 	INIT_GMP_NUM(gmpnum_result2);
 
@@ -949,7 +1243,7 @@ ZEND_FUNCTION(gmp_gcd)
 		WRONG_PARAM_COUNT;
 	}
 
-	gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_gcd, (gmp_binary_ui_op_t)mpz_gcd_ui, 1 TSRMLS_CC);
+	gmp_zval_binary_ui_op_ex(return_value, a_arg, b_arg, mpz_gcd, (gmp_binary_ui_op_t)mpz_gcd_ui, 1, 0 TSRMLS_CC);
 }
 /* }}} */
 
@@ -1102,7 +1396,7 @@ ZEND_FUNCTION(gmp_random)
 
 		GMPG(rand_initialized) = 1;
 	}
-	mpz_urandomb(*gmpnum_result, GMPG(rand_state), GMP_ABS (limiter) * __GMP_BITS_PER_MP_LIMB);
+	mpz_urandomb(*gmpnum_result, GMPG(rand_state), GMP_ABS (limiter) * GMP_LIMB_BITS);
 
 	ZEND_REGISTER_RESOURCE(return_value, gmpnum_result, le_gmp);
 }
@@ -1132,6 +1426,14 @@ ZEND_FUNCTION(gmp_com)
 }
 /* }}} */
 
+/* {{{ proto resource gmp_nextprime(resource a)
+   Finds next prime of a */
+ZEND_FUNCTION(gmp_nextprime)
+{
+   gmp_unary_op(mpz_nextprime);
+}
+/* }}} */
+
 /* {{{ proto resource gmp_xor(resource a, resource b)
    Calculates logical exclusive OR of a and b */
 ZEND_FUNCTION(gmp_xor)
@@ -1190,6 +1492,11 @@ ZEND_FUNCTION(gmp_setbit)
 			break;
 	}
 
+	if (index < 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Index must be greater than or equal to zero");
+		return;
+	}
+
 	if (set) {
 		mpz_setbit(*gmpnum_a, index);
 	} else {
@@ -1214,6 +1521,11 @@ ZEND_FUNCTION(gmp_clrbit)
 
 	convert_to_long_ex(ind_arg);
 	index = Z_LVAL_PP(ind_arg);
+	
+	if (index < 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Index must be greater than or equal to zero");
+		return;
+	}
 
 	mpz_clrbit(*gmpnum_a, index);
 }
@@ -1268,6 +1580,11 @@ ZEND_FUNCTION(gmp_scan0)
 	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
 	convert_to_long_ex(start_arg);
 
+	if (Z_LVAL_PP(start_arg) < 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Starting index must be greater than or equal to zero");
+		RETURN_FALSE;
+	}
+
 	RETURN_LONG(mpz_scan0(*gmpnum_a, Z_LVAL_PP(start_arg)));
 }
 /* }}} */
@@ -1285,6 +1602,11 @@ ZEND_FUNCTION(gmp_scan1)
 
 	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
 	convert_to_long_ex(start_arg);
+	
+	if (Z_LVAL_PP(start_arg) < 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Starting index must be greater than or equal to zero");
+		RETURN_FALSE;
+	}
 
 	RETURN_LONG(mpz_scan1(*gmpnum_a, Z_LVAL_PP(start_arg)));
 }
@@ -1307,6 +1629,6 @@ static void _php_gmpnum_free(zend_rsrc_list_entry *rsrc TSRMLS_DC)
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: sw=4 ts=4 fdm=marker
- * vim<600: sw=4 ts=4
+ * vim600: noet sw=4 ts=4 fdm=marker
+ * vim<600: noet sw=4 ts=4
  */
