@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | PHP Version 5                                                        |
   +----------------------------------------------------------------------+
-  | Copyright (c) 1997-2007 The PHP Group                                |
+  | Copyright (c) 1997-2010 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -17,11 +17,12 @@
   |          Dmitry Stogov <dmitry@zend.com>                             |
   +----------------------------------------------------------------------+
 */
-/* $Id: php_http.c,v 1.77.2.11.2.8 2007/02/24 02:17:26 helly Exp $ */
+/* $Id: php_http.c 304084 2010-10-05 11:43:59Z dmitry $ */
 
 #include "php_soap.h"
 #include "ext/standard/base64.h"
 #include "ext/standard/md5.h"
+#include "ext/standard/php_rand.h"
 
 static char *get_http_header_value(char *headers, char *type);
 static int get_http_body(php_stream *socketd, int close, char *headers,  char **response, int *out_size TSRMLS_DC);
@@ -32,7 +33,7 @@ static int get_http_headers(php_stream *socketd,char **response, int *out_size T
 
 static int stream_alive(php_stream *stream  TSRMLS_DC)
 {
-	long socket;
+	int socket;
 	char buf;
 
 	/* maybe better to use:
@@ -166,6 +167,13 @@ static php_stream* http_connect(zval* this_ptr, php_url *phpurl, int use_ssl, in
 		smart_str_appendc(&soap_headers, ':');
 		smart_str_append_unsigned(&soap_headers, phpurl->port);
 		smart_str_append_const(&soap_headers, " HTTP/1.1\r\n");
+		smart_str_append_const(&soap_headers, "Host: ");
+		smart_str_appends(&soap_headers, phpurl->host);
+		if (phpurl->port != 80) {
+			smart_str_appendc(&soap_headers, ':');
+			smart_str_append_unsigned(&soap_headers, phpurl->port);
+		}
+		smart_str_append_const(&soap_headers, "\r\n");
 		proxy_authentication(this_ptr, &soap_headers TSRMLS_CC);
 		smart_str_append_const(&soap_headers, "\r\n");
 		if (php_stream_write(stream, soap_headers.c, soap_headers.len) != soap_headers.len) {
@@ -236,9 +244,11 @@ int make_http_soap_request(zval  *this_ptr,
 	int http_1_1;
 	int http_status;
 	int content_type_xml = 0;
+	long redirect_max = 20;
 	char *content_encoding;
 	char *http_msg = NULL;
 	zend_bool old_allow_url_fopen;
+	php_stream_context *context = NULL;
 
 	if (this_ptr == NULL || Z_TYPE_P(this_ptr) != IS_OBJECT) {
 		return FALSE;
@@ -287,6 +297,7 @@ int make_http_soap_request(zval  *this_ptr,
 				request_size = Z_STRLEN(retval);
 			} else {
 				if (request != buf) {efree(request);}
+				smart_str_free(&soap_headers_z);
 				return FALSE;
 			}
 	  }
@@ -305,11 +316,25 @@ int make_http_soap_request(zval  *this_ptr,
 		phpurl = php_url_parse(location);
 	}
 
+	if (SUCCESS == zend_hash_find(Z_OBJPROP_P(this_ptr),
+			"_stream_context", sizeof("_stream_context"), (void**)&tmp)) {
+		context = php_stream_context_from_zval(*tmp, 0);
+	}
+
+	if (context && 
+		php_stream_context_get_option(context, "http", "max_redirects", &tmp) == SUCCESS) {
+		if (Z_TYPE_PP(tmp) != IS_STRING || !is_numeric_string(Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp), &redirect_max, NULL, 1)) {
+			if (Z_TYPE_PP(tmp) == IS_LONG)
+				redirect_max = Z_LVAL_PP(tmp);
+		}
+	}
+
 try_again:
 	if (phpurl == NULL || phpurl->host == NULL) {
 	  if (phpurl != NULL) {php_url_free(phpurl);}
 		if (request != buf) {efree(request);}
 		add_soap_fault(this_ptr, "HTTP", "Unable to parse URL", NULL, NULL TSRMLS_CC);
+		smart_str_free(&soap_headers_z);
 		return FALSE;
 	}
 
@@ -320,6 +345,7 @@ try_again:
 		php_url_free(phpurl);
 		if (request != buf) {efree(request);}
 		add_soap_fault(this_ptr, "HTTP", "Unknown protocol. Only http and https are allowed.", NULL, NULL TSRMLS_CC);
+		smart_str_free(&soap_headers_z);
 		return FALSE;
 	}
 
@@ -330,6 +356,7 @@ try_again:
 		if (request != buf) {efree(request);}
 		add_soap_fault(this_ptr, "HTTP", "SSL support is not available in this build", NULL, NULL TSRMLS_CC);
 		PG(allow_url_fopen) = old_allow_url_fopen;
+		smart_str_free(&soap_headers_z);
 		return FALSE;
 	}
 
@@ -379,6 +406,7 @@ try_again:
 			if (request != buf) {efree(request);}
 			add_soap_fault(this_ptr, "HTTP", "Could not connect to host", NULL, NULL TSRMLS_CC);
 			PG(allow_url_fopen) = old_allow_url_fopen;
+			smart_str_free(&soap_headers_z);
 			return FALSE;
 		}
 	}
@@ -432,12 +460,15 @@ try_again:
 				smart_str_appendl(&soap_headers, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
 				smart_str_append_const(&soap_headers, "\r\n");
 			}
-		} else{
+		} else if (FG(user_agent)) {
+			smart_str_append_const(&soap_headers, "User-Agent: ");
+			smart_str_appends(&soap_headers, FG(user_agent));
+			smart_str_append_const(&soap_headers, "\r\n");
+		} else {
 			smart_str_append_const(&soap_headers, "User-Agent: PHP-SOAP/"PHP_VERSION"\r\n");
 		}
 
 		smart_str_append(&soap_headers, &soap_headers_z);
-		smart_str_free(&soap_headers_z);
 
 		if (soap_version == SOAP_1_2) {
 			smart_str_append_const(&soap_headers,"Content-Type: application/soap+xml; charset=utf-8");
@@ -469,10 +500,9 @@ try_again:
 					char          HA1[33], HA2[33], response[33], cnonce[33], nc[9];
 					PHP_MD5_CTX   md5ctx;
 					unsigned char hash[16];
-					unsigned int ctx;
 
 					PHP_MD5Init(&md5ctx);
-					snprintf(cnonce, sizeof(cnonce), "%d", php_rand_r(&ctx));
+					snprintf(cnonce, sizeof(cnonce), "%ld", php_rand(TSRMLS_C));
 					PHP_MD5Update(&md5ctx, (unsigned char*)cnonce, strlen(cnonce));
 					PHP_MD5Final(hash, &md5ctx);
 					make_digest(cnonce, hash);
@@ -606,6 +636,11 @@ try_again:
 						smart_str_append_const(&soap_headers, "\", opaque=\"");
 						smart_str_appendl(&soap_headers, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
 					}
+					if (zend_hash_find(Z_ARRVAL_PP(digest), "algorithm", sizeof("algorithm"), (void **)&tmp) == SUCCESS &&
+						Z_TYPE_PP(tmp) == IS_STRING) {
+						smart_str_append_const(&soap_headers, "\", algorithm=\"");
+						smart_str_appendl(&soap_headers, Z_STRVAL_PP(tmp), Z_STRLEN_PP(tmp));
+					}
 					smart_str_append_const(&soap_headers, "\"\r\n");
 				}
 			} else {
@@ -683,18 +718,18 @@ try_again:
 		err = php_stream_write(stream, soap_headers.c, soap_headers.len);
 		if (err != soap_headers.len) {
 			if (request != buf) {efree(request);}
-			smart_str_free(&soap_headers);
 			php_stream_close(stream);
 			zend_hash_del(Z_OBJPROP_P(this_ptr), "httpurl", sizeof("httpurl"));
 			zend_hash_del(Z_OBJPROP_P(this_ptr), "httpsocket", sizeof("httpsocket"));
 			zend_hash_del(Z_OBJPROP_P(this_ptr), "_use_proxy", sizeof("_use_proxy"));
 			add_soap_fault(this_ptr, "HTTP", "Failed Sending HTTP SOAP request", NULL, NULL TSRMLS_CC);
+			smart_str_free(&soap_headers_z);
 			return FALSE;
 		}
 		smart_str_free(&soap_headers);
-
 	} else {
 		add_soap_fault(this_ptr, "HTTP", "Failed to create stream??", NULL, NULL TSRMLS_CC);
+		smart_str_free(&soap_headers_z);
 		return FALSE;
 	}
 
@@ -702,6 +737,7 @@ try_again:
 		php_stream_close(stream);
 		zend_hash_del(Z_OBJPROP_P(this_ptr), "httpsocket", sizeof("httpsocket"));
 		zend_hash_del(Z_OBJPROP_P(this_ptr), "_use_proxy", sizeof("_use_proxy"));
+		smart_str_free(&soap_headers_z);
 		return TRUE;
 	}
 
@@ -713,6 +749,7 @@ try_again:
 			zend_hash_del(Z_OBJPROP_P(this_ptr), "httpsocket", sizeof("httpsocket"));
 			zend_hash_del(Z_OBJPROP_P(this_ptr), "_use_proxy", sizeof("_use_proxy"));
 			add_soap_fault(this_ptr, "HTTP", "Error Fetching http headers", NULL, NULL TSRMLS_CC);
+			smart_str_free(&soap_headers_z);
 			return FALSE;
 		}
 
@@ -861,6 +898,7 @@ try_again:
 		if (http_msg) {
 			efree(http_msg);
 		}
+		smart_str_free(&soap_headers_z);
 		return FALSE;
 	}
 
@@ -910,16 +948,24 @@ try_again:
 				efree(http_body);
 				efree(loc);
 				if (new_url->scheme == NULL && new_url->path != NULL) {
-					new_url->scheme = NULL;
+					new_url->scheme = phpurl->scheme ? estrdup(phpurl->scheme) : NULL;
 					new_url->host = phpurl->host ? estrdup(phpurl->host) : NULL;
 					new_url->port = phpurl->port;
 					if (new_url->path && new_url->path[0] != '/') {
-						char *t = phpurl->path;
-						char *p = strrchr(t, '/');
-						if (p) {
-							char *s = emalloc((p - t) + strlen(new_url->path) + 2);
-							strncpy(s, t, (p - t) + 1);
-							s[(p - t) + 1] = 0;
+						if (phpurl->path) {
+							char *t = phpurl->path;
+							char *p = strrchr(t, '/');
+							if (p) {
+								char *s = emalloc((p - t) + strlen(new_url->path) + 2);
+								strncpy(s, t, (p - t) + 1);
+								s[(p - t) + 1] = 0;
+								strcat(s, new_url->path);
+								efree(new_url->path);
+								new_url->path = s;
+							} 
+						} else {
+							char *s = emalloc(strlen(new_url->path) + 2);
+							s[0] = '/'; s[1] = 0;
 							strcat(s, new_url->path);
 							efree(new_url->path);
 							new_url->path = s;
@@ -928,6 +974,12 @@ try_again:
 				}
 				phpurl = new_url;
 
+				if (--redirect_max < 1) {
+					add_soap_fault(this_ptr, "HTTP", "Redirection limit reached, aborting", NULL, NULL TSRMLS_CC);
+					smart_str_free(&soap_headers_z);
+					return FALSE;
+				}
+
 				goto try_again;
 			}
 		}
@@ -1008,6 +1060,7 @@ try_again:
 		}
 		if (auth) efree(auth);
 	}
+	smart_str_free(&soap_headers_z);
 
 	/* Check and see if the server even sent a xml document */
 	content_type = get_http_header_value(http_headers,"Content-Type: ");
@@ -1144,17 +1197,19 @@ static char *get_http_header_value(char *headers, char *type)
 
 			/* match */
 			tmp = pos + typelen;
-			eol = strstr(tmp, "\r\n");
+			eol = strchr(tmp, '\n');
 			if (eol == NULL) {
 				eol = headers + headerslen;
+			} else if (eol > tmp && *(eol-1) == '\r') {
+				eol--;
 			}
 			return estrndup(tmp, eol - tmp);
 		}
 
 		/* find next line */
-		pos = strstr(pos, "\r\n");
+		pos = strchr(pos, '\n');
 		if (pos) {
-			pos += 2;
+			pos++;
 		}
 
 	} while (pos);
@@ -1194,7 +1249,7 @@ static int get_http_body(php_stream *stream, int close, char *headers,  char **r
 	}
 
 	if (header_chunked) {
-		char done, chunk_size[10];
+		char ch, done, chunk_size[10], headerbuf[8192];
 
 		done = FALSE;
 
@@ -1222,11 +1277,20 @@ static int get_http_body(php_stream *stream, int close, char *headers,  char **r
 						len_size += len_read;
 	 					http_buf_size += len_read;
  					}
- 				}
 
-				/* Eat up '\r' '\n' */
-				php_stream_getc(stream);
-				php_stream_getc(stream);
+					/* Eat up '\r' '\n' */
+					ch = php_stream_getc(stream);
+					if (ch == '\r') {
+						ch = php_stream_getc(stream);
+					}
+					if (ch != '\n') {
+						/* Somthing wrong in chunked encoding */
+						if (http_buf) {
+							efree(http_buf);
+						}
+						return FALSE;
+					}
+ 				}
 			} else {
 				/* Somthing wrong in chunked encoding */
 				if (http_buf) {
@@ -1239,6 +1303,19 @@ static int get_http_body(php_stream *stream, int close, char *headers,  char **r
 			}
 		}
 
+		/* Ignore trailer headers */
+		while (1) {
+			if (!php_stream_gets(stream, headerbuf, sizeof(headerbuf))) {
+				break;
+			}
+
+			if ((headerbuf[0] == '\r' && headerbuf[1] == '\n') ||
+			    (headerbuf[0] == '\n')) {
+				/* empty line marks end of headers */
+				break;
+			}
+		}
+
 		if (http_buf == NULL) {
 			http_buf = emalloc(1);
 		}
@@ -1285,7 +1362,8 @@ static int get_http_headers(php_stream *stream, char **response, int *out_size T
 			break;
 		}
 
-		if (strcmp(headerbuf, "\r\n") == 0) {
+		if ((headerbuf[0] == '\r' && headerbuf[1] == '\n') ||
+		    (headerbuf[0] == '\n')) {
 			/* empty line marks end of headers */
 			done = TRUE;
 			break;
